/* sb.c */

/* VIS-5D version 3.0 */

/*
vis5d program for visualizing five dimensional gridded data sets
Copyright (C) 1990, 1991, 1992  Bill Hibbard, Brian Paul, Dave Santek,
and Andre Battaiola.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 1, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Spaceball support */


#include    <stdio.h>
#include    <math.h>
#ifdef stellar
#  define     Sb_ENVIRONMENT_INCLUDES
#  include    <sblibry.h>     /* defines Spaceball library names */
#endif
#ifdef sgi
#  include "x.h"
#  include <gl/spaceball.h>
#endif

#include "globals.h"
#include "matrix.h"



#ifdef stellar
   static int  spaceball_present = -1;

   /* change these values for faster/slower reaction: */
   static float  translation_rate = .001,
                 rotation_rate = .01,
                 zoom_rate = .0005;
   static int movement = 0;
#endif




/*** initialize_spaceball *********************************************
   Initialize the spaceball.
   Return:  0 - spaceball not present or error.
            1 - spaceball ready.
**********************************************************************/
int initialize_spaceball()
{
#if defined (stellar)
   char *spaceball_tty;
   int mypressed(), mytranslate(), mydelta(), myabsolute();

   spaceball_tty = (char *) getenv( "SPACEBALL" );
   if (spaceball_tty == NULL) {
      return -1;
   }

   /* look for the spaceball_tty */
   spaceball_present = SbOpen( spaceball_tty );

   if (spaceball_present != -1) {

      /* reset Spaceball and wait for it to respond */
      if (SbReset( TRUE ) != -1) {
	 SbSetBallMode( "SSD" );     /* select the initial ball mode */
	 SbFunctions.delta = mydelta;
	 SbFunctions.pressed = mypressed;
	 SbFunctions.translate = mytranslate;
      }
      else {
	 fprintf(stderr,"Spaceball not responding on %s\n",spaceball_tty);
	 SbClose();
	 spaceball_present = -1;
      }
   }

   if (spaceball_present==-1)
     return 0;
   else
     return 1;
#elif defined (sgi)
   return sbexists();
#else
   return 0;
#endif
}



/*** close_spaceball **************************************************
   Call when finished with Spaceball.
**********************************************************************/
void close_spaceball()
{
#ifdef stellar
   if (spaceball_present!=-1)
      SbClose();
#endif
}



/*** prompt_spaceball *************************************************
   Prompt the spaceball for input.
**********************************************************************/
void prompt_spaceball()
{
#ifdef stellar
   if (spaceball_present==-1)  return;
   if (!SbBallDataPending)     /* has a request been made? */
     SbRequestBallData();    /* will set SbBallDataPending TRUE */
   movement = 0;
#endif
}





/*** check_spaceball **************************************************
   Check for spaceball input, if any modify the CTM and CTMinv
   accordingly.
   Return:  0 if no movement
            1 if movement
**********************************************************************/
int check_spaceball()
{
#ifdef stellar
   if (spaceball_present==-1)  return;

   if (!SbBallDataPending) {
      fprintf( stderr,
"A call to prompt_spaceball() should have been made prior\n\
to waiting for input\n"
	      );
      prompt_spaceball();     /* ensure a request has been sent */
   }

   SbCheckSpaceball();         /* sets SbBallDataPending FALSE when a
                                 * data packet is received */
   if (!SbBallDataPending) {

      /* if the whole loop is taking less than 1/5th second then
       * request the ball data now otherwise request just prior to
       * waiting for input with a call to prompt_spaceball() */

      if (SbAveragePeriod < (200 * 4))    /* 200msecs * 4 = 1/5th sec */
	  SbRequestBallData();
   }
   return movement;
#endif
}



#ifdef stellar

static int mypressed( keys )
SbUint16 keys;
{

   if (keys&0x01)   /* function key 1 = Top view*/ 
     make_matrix( 0.0, 0.0, 0.0, DEFAULT_SCALE, 0.0, 0.0, 0.0, CTM );
   if (keys&0x02)   /* key 2 = South */
     make_matrix( 90.0, 0.0, 0.0, DEFAULT_SCALE, 0.0, 0.0, 0.0, CTM );
   if (keys&0x04)   /* key 3 = West */
     make_matrix( 90.0, -90.0, 0.0, DEFAULT_SCALE, 0.0, 0.0, 0.0, CTM );
   if (keys&0x10)   /* key 5 = Bottom */
     make_matrix( 0.0, 180.0, 0.0, DEFAULT_SCALE, 0.0, 0.0, 0.0, CTM ); 
   if (keys&0x20)   /* key 6 = North */
     make_matrix( 90.0, 180.0, 0.0, DEFAULT_SCALE, 0.0, 0.0, 0.0, CTM );
   if (keys&0x40)   /* key 7 = East */
     make_matrix( 90.0, 90.0, 0.0, DEFAULT_SCALE, 0.0, 0.0, 0.0, CTM );
   if (keys&0x80)   /* key 8 = Reset SpaceBall */
      SbRezero();
   movement = 1;
}


static int mytranslate( period, vec )
SbUint16 period;
SbReal vec[ 3 ];
{
   float scale, transx, transy;
   Matrix mat;

   transx = vec[0] * period * translation_rate;
   transy = vec[1] * period * translation_rate;
   scale = 1.0 - (vec[2] * period * zoom_rate);

   make_matrix( 0.0,0.0,0.0, scale, transx,transy,0.0, mat );
   mat_mul( CTM, CTM, mat );
   mat_inv( CTMinv, CTM );
   movement = 1;
}


static int mydelta( delta_rotation_matrix )
SbMatrixType delta_rotation_matrix;
{
   Matrix delta_4x4;
   int i,j;

   for (i=0;i<3;i++) {
      for (j=0;j<3;j++)
          delta_4x4[i][j] = delta_rotation_matrix[i][j];
      delta_4x4[i][3] = 0.0;
      delta_4x4[3][i] = 0.0;
   }
   delta_4x4[3][3] = 1.0;

   mat_mul( CTM, CTM, delta_4x4 );
   mat_inv( CTMinv, CTM );
   movement = 1;
}

#endif
